# (C) 2018 - TECHNISCHE UNIVERSITEIT EINDHOVEN
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED,INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS
# FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE
# COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE FOR ANY DAMAGES
# OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT
# OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
# THIS SOFTWARE IS PART OF THE TU/e COMPUTER SCIENCE REPORT 2018-01
# Citation for published version:
#
# Sanchez, L., Wesselink, J.W., & Willemse, T.A.C. (2018). BDD-based parity game solving: a comparison of
# Zielonka's recursive algorithm, priority promotion and fixpoint iteration. (Computer science reports; Vol. 1801).
# Eindhoven: Technische Universiteit Eindhoven.
# https://pure.tue.nl/ws/files/92755535/CSR_18_01.pdf



import configBDD
import solver_library
import generate_random_game
from time import process_time
import sys, os, random, math

debug = False
RANGE = range(3,35)
REPEAT_EXPERIMENTS = 5000

#generate games with size (nr. Booleans) in range(3,35):
for Size_of_Games in RANGE:
  zielonka_total_time = 0.0
  pp_total_time = 0.0
  solved_games =0
  PrioLimit = math.floor(math.sqrt(2*Size_of_Games))
  print()
  print('Solving games with %d variables and at most %d priorities' %(Size_of_Games, PrioLimit))
  for iteration in range(0, REPEAT_EXPERIMENTS):
    (variables, variables_, V, Edges, Even, Odd, Priorities) = generate_random_game.random_game(Size_of_Games,PrioLimit,4,False,False)
    g = solver_library.PG (variables,variables_, V, Edges, Even, Odd, Priorities)
    h = g. duplicate()
    configBDD.bdd. collect_garbage()

    # solve game using zielonka
    zielonka_solver_start = process_time()
    (W0,W1) = g. zielonka (debug)
    solved_games += 1
    zielonka_solver_end = process_time()
    zielonka_total_time = zielonka_total_time + (zielonka_solver_end - zielonka_solver_start)
    configBDD.bdd. collect_garbage()

    # solve game using PP
    pp_solver_start = process_time()
    (W0_,W1_) = h. PP (debug)
    pp_solver_end = process_time()
    pp_total_time = pp_total_time + (pp_solver_end - pp_solver_start)
    configBDD.bdd. collect_garbage()
    print('                                                      iteration: %d.  Time: %d (Zlk) and %d (PP)\r' %(solved_games, zielonka_total_time, pp_total_time),end='')
    if zielonka_total_time < pp_total_time:
      print('[%d vars, <= %d prio] Zlk is winning by %d sec.\r'%(Size_of_Games,PrioLimit, pp_total_time -zielonka_total_time),end='')
    else:
      print('[%d vars, <= %d prio] PP is winning by %d sec.\r'%(Size_of_Games,PrioLimit, zielonka_total_time -pp_total_time),end='')

    # sanity checks
    if W0 != W0_:
      print('hmmm...; according to PP, %d vertices are won by Even and %d by Odd' %(configBDD.bdd.count(W0_,len(h.variables)),configBDD.bdd.count(W1_,len(h.variables))))
      print('        according to Zlk, %d vertices are won by Even and %d by Odd' %(configBDD.bdd.count(W0,len(h.variables)),configBDD.bdd.count(W1,len(h.variables))))
      print(' solving PP even dominion using Zlk')
      k = g.duplicate()
      k. remove(W1_)
      (X0,X1) = k. zielonka()
      print('        according to Zlk, %d vertices of the even partition are won by Even and %d by Odd' %(configBDD.bdd.count(X0,len(h.variables)),configBDD.bdd.count(X1,len(h.variables))))
      print(' Solving subgame again but now using PP')
      (X0,X1) = k. PP()
      print('hmmm...; according to PP, %d vertices are won by Even and %d by Odd' %(configBDD.bdd.count(X0,len(h.variables)),configBDD.bdd.count(X1,len(h.variables))))
      print('Printing offending graph with debug info')
      g. info(True)
      sys. exit()

